// KISS Compiler Version 0.1
// Distributed under GPL
// Author: Antonio Bonifati

// Notes: module to handle Boolean expression (Boolean parser),
// that is conditions with/without relations.
// Generated code is unoptimized: must be enhanced.
// (for example short evaluation should be performed).


/*------------------------------------------------------------*/
                          /* Boolexpr */


#include "boolexpr.h"
#include "codegen.h"
#include "scanner.h"
#include "recogn.h"
#include "expr.h"


/*------------------------------------------------------------*/
/* Recognize and Translate a Relational "Equals" */

void Equals()
{
   Match('=');
   Expression();

   PopCompare();                // CODE emission

   /** puts 0 into AX if ZF=0, else puts -1 **/

   SetEqual();                  // CODE emission
}


/*------------------------------------------------------------*/
/* Recognize and Translate a Relational "Not Equals" */

void NotEquals()
{
   Match('>');
   Expression();

   PopCompare();                // CODE emission

   SetNEqual();                 // CODE emission
}

/*------------------------------------------------------------*/
/* Recognize and Translate a Relational "Less Than or Equal" */

void LessOrEqual()
{
   Match('=');
   Expression();

   PopCompare();                // CODE emission
   
   SetLessOrEqual();            // CODE emission
}


/*------------------------------------------------------------*/
/* Recognize and Translate a Relational "Less Than" */

void Less()
{
   Match('<');
   switch(Look)
   {
      case '=': LessOrEqual();  break;
      case '>': NotEquals();    break;
      
      default: Expression();
      
               PopCompare();    // CODE emission
               SetLess();       // CODE emission
               
               break;
   }
}


/*------------------------------------------------------------*/
/* Recognize and Translate a Relational "Greater Than" */

void Greater()
{
   Match('>');
   if (Look =='=')
   {
      Match('=');
      Expression();
      
      PopCompare();             // CODE emission
      
      SetGreaterOrEqual();      // CODE emission
   }
   else
   {
      Expression();
      
      PopCompare();             // CODE emission
      
      SetGreater();             // CODE emission
   }
}


/*------------------------------------------------------------*/
/* Parse and Translate a Relation                             */
/*                                                            */
/* <relation> ::= <expression> [<relop> <expression>]         */

void Relation()
{
   Expression();
   if (IsRelop(Look))
   {
      Push();        // CODE emission

      switch(Look)
      {
         case '=': Equals();    break;
         case '#': NotEquals(); break;
         case '<': Less();      break;
         case '>': Greater();   break;
      }

      /* here condition flags have to be setted
         according to the boolean value
         of the relation stored in accumulator.
         Test() takes care of emitting
         code (if any) for this */

      Test();           // CODE emission
   }
}


/*------------------------------------------------------------*/
/* Parse and Translate a Boolean Factor with NOT              */
/*                                                            */
/* <b-factor> ::= <b-literal> | <b-variable> | <relation>     */
// this will be in KISS
//void BoolFactor()
//{
//   if (IsBoolean(Look))
//      if (GetBoolean())
//         LoadTRUE();       // CODE emission
//      else
//         LoadFALSE();      // CODE emission
//   else
//      Relation();
//}


/*------------------------------------------------------------*/
/* Parse and Translate a Boolean Factor with NOT              */
/*                                                            */
/* <not-factor>   ::= [!] <relation>                          */

void NotFactor()
{
   if (Look == '!')
   {
      Match('!');
      /* in KISS use BoolFactor(); */
      Relation();

      Not();            // CODE emission
   }   
   else
      /* in KISS use BoolFactor(); */   
      Relation();
}


/*------------------------------------------------------------*/
/* Parse and Translate a Boolean Term                         */
/*                                                            */
/* <b-term> ::= <not-factor> [AND <not-factor>]*              */

void BoolTerm()
{
   NotFactor();
   while (Look == '&')
   {
      Push();           // CODE emission

      Match('&');
      NotFactor();

      PopAnd();         // CODE emission
   }
}


/*------------------------------------------------------------*/
/* Recognize and Translate a Boolean OR */

void BoolOr()
{
   Match('|');
   BoolTerm();

   PopOr();             // CODE emission
}


/*------------------------------------------------------------*/
/* Recognize and Translate an Exclusive Or */

void BoolXor()
{
   Match('~');
   BoolTerm();

   PopXor();            // CODE emission
}


/*------------------------------------------------------------*/
/* Parse and Translate a Boolean Expression (or Condition)    */
/*                                                            */
/* <b-expression> ::= <b-term> [<orop> <b-term>]*             */

void BoolExpression()
{
   BoolTerm();
   while (IsOrOp(Look))
   {
      Push();           // CODE emission

      switch(Look)
      {
         case '|': BoolOr();     break;
         case '~': BoolXor();    break;
      }
   }
}

