// KISS Compiler Version 0.1
// Distributed under GPL
// Author: Antonio Bonifati

// Notes: module to handle math expressions (arithmetic parser).
// Assignments, Relations and conditions (boolean parser)
// are handled by boolexpr.cpp and stat.cpp
// Generated code is unoptimized: must be enhanced.

/*------------------------------------------------------------*/
                          /* Expr */

#include "expr.h"
#include "boolexpr.h"
//#include "main.h"
#include "recogn.h"
#include "scanner.h"
#include "codegen.h"
#include "prog.h"
#include "error.h"


// will be used in KISS
/*------------------------------------------------------------*/
/* Parse and Translate an Identifier                          */
/*                                                            */
/* <ident> ::= <name> [( )]                                   */

//void Ident()
//{
//   GetName();

   // ignore extra chars in names longer than IDMAXLEN
//   string Name(Value, 0, IDMAXLEN);

//   if (InTable(Name)==NOTFOUND) Undefined(Name);
   
//   if (Look == '(')
//   {
//      Match('(');
//      Match(')');

//      CallProc(Name);   // CODE emission
//   }
//   else
//      LoadVar(Name); // CODE emission
//}

/*------------------------------------------------------------*/
/* Parse and Translate a Math Factor                          */
/*                                                            */
/* <factor> ::= <integer> | ( <b-expression> ) | <var>        */

void Factor()
{
   NewLine();
   Comment();
   if (Look == '(')
   {
      Match('(');
      BoolExpression();
      Match(')');
   }
   else if (isalpha(Look))
   {
      GetName();
      string Name(Value, 0, IDMAXLEN);

      if (InTable(Name)==NOTFOUND) Undefined(Name);

      LoadVar(Name); // CODE emission
      
      // replace with this in KISS
      //Ident();
   }
   else
   {
      GetNum();

      LoadConst(Value);      // CODE emission
   }
}


/*------------------------------------------------------------*/
/* Parse and Translate a Negative Factor */

void NegFactor()
{
   Match('-');
   if (isdigit(Look))
   {
      GetNum();

      LoadNegConst(Value);      // CODE emission
   }
   else
   {
      Factor();
      
      Negate();                 // CODE emission
   }
}


/*------------------------------------------------------------*/
/* Parse and Translate the First Math Factor                  */
/*                                                            */
/* <first factor>::= [<addop>] <factor>                       */

void FirstFactor()
{
   NewLine();
   Comment();
   switch(Look)
   {
      case '+':
           Match('+');
           Factor();
           break;
      case '-': NegFactor();
           break;
      default: Factor();
   }
}

/*------------------------------------------------------------*/
/* Recognize and Translate a Multiply                         */

void Multiply()
{
   Match('*');
   Factor();

   PopMul();      // CODE emission
}

/*-----------------------------------------------------------*/
/* Recognize and Translate a Divide                          */

void Divide()
{
   Match('/');
   Factor();

   PopDiv();      // CODE emission
}

/*-----------------------------------------------------------*/
/* Recognize and Translate a Modulus                         */

void Modulus()
{
   Match('%');
   Factor();

   PopMod();      // CODE emission
}

/*---------------------------------------------------------------*/
/* Completion of Term Processing  (called by Term and FirstTerm) */
/*                                                               */
/* <rest> ::= [<mulop> <factor>]*                                */

void Term1()
{
   while (IsMulop(Look))
   {
      Push();        // CODE emission

      switch(Look)
      {
        case '*': Multiply(); break;
        case '/': Divide();   break;
        case '%': Modulus();  break;
      }
   }
}


/*------------------------------------------------------------*/
/* Parse and Translate an Expression Term                     */
/*                                                            */
/* <term> ::= <factor> <rest>                                 */

void Term()
{
   Factor();
   Term1();
}


/*------------------------------------------------------------*/
/* Parse and Translate a Math Term with Possible Leading Sign */
/*                                                            */
/* <first term> ::= <first factor> <rest>                     */

void FirstTerm()
{
   FirstFactor();
   Term1();
}


/*------------------------------------------------------------*/
/* Recognize and Translate an Add */

void Add()
{
   Match('+');
   Term();

   PopAdd();            // CODE emission
}


/*------------------------------------------------------------*/
/* Recognize and Translate a Subtract */

void Subtract()
{
   Match('-');
   Term();

   PopSub();            // CODE emission
}


/*------------------------------------------------------------*/
/* Parse and Translate a Math Expression                      */
/*                                                            */
/* <expression> ::= <firstterm> [<addop> <term>]*             */

void Expression()
{
   FirstTerm();
   while (IsAddop(Look))
   {
      Push();        // CODE emission

      switch(Look)
      {
        case '+': Add();      break;
        case '-': Subtract(); break;
      }
   }
}

