// KISS Compiler Version 0.1
// Distributed under GPL
// Author: Antonio Bonifati

// Notes: This module contains only symbol recognition routines.

/*------------------------------------------------------------*/
                          /* Recogn */

#include "main.h"
#include "recogn.h"


/*------------------------------------------------------------*/
/* Recognize White Space (NewLine and similar excluded)       */
/* <white> ::= ' ' | TAB | '\r'                               */

bool IsWhite(char c)
{
   return(c==' ' || c==TAB || c=='\r');
}

/*------------------------------------------------------------*/
/* Recognize a Newline and similar characters                 */
/* that bring to a new line (form feed, vertical tab)         */
/* <newline> ::= '\n' | '\f' | '\v'                           */
bool IsLineSep(char c) /* recognize all line-separator        */
{
   return(c==NEWLINE || c=='\f' || c=='\v');
}

/*------------------------------------------------------------*/
/* Recognize an Alpha Character                               */
/*                                                            */
/* <letter> ::= a|b|...|z|A|B|...|Y|Z                         */

/* int isalpha(int c);   defined in <ctype.h> */


/*------------------------------------------------------------*/
/* Recognize a Decimal Digit                                  */
/*                                                            */
/* <digit> ::= 0|1|...|8|9                                    */

/* int isdigit(int c);   defined in <ctype.h> */


/*------------------------------------------------------------*/
/* Recognize an Alphanumeric                                  */
/* <alnum> := <letter> | <digit>                              */

/* int isalnum(int c);   defined in <ctype.h> */


/*------------------------------------------------------------*/
/* Recognize an Addop                                         */
/*                                                            */
/* <addop> ::= +|-                                            */

bool IsAddop(char c)
{
   return(c=='+' || c=='-');
}


/*------------------------------------------------------------*/
/* Recognize an Mulop                                         */
/*                                                            */
/* <mulop> ::= * | / | %                                      */

bool IsMulop(char c)
{
   return(c=='*' || c=='/' || c=='%');
}


/*------------------------------------------------------------*/
/* Recognize a Boolean Literal                                */
/*                                                            */
/* <b-literal> ::= TRUE | FALSE                               */
// used only in KISS
// bool IsBoolean(char c)
// {
//    return((c=toupper(c))=='T' || c=='F');
// }

/*------------------------------------------------------------*/
/* Recognize a Boolean Orop                                   */
/*                                                            */
/* <orop> ::= OR | "|"   |   XOR | ~                          */

bool IsOrOp(char c)
{
   return(c=='|'  ||  c=='~');
}

/*------------------------------------------------------------*/
/* Recognize a Relop                                          */
/*                                                            */
/* <relop> ::= <> | = | < | > | <= | >=                       */

bool IsRelop(char c)
{
   return(c=='='  ||  c=='<'  ||  c=='>');
}

/*------------------------------------------------------------*/
/* Recognize Any Operator */

/* you DON'T have to include every possible operator in this list.
   For example, the paretheses aren't included, nor is the terminating
   period. The current version of Scan handles single-character operators
   just fine as it is. The list above includes only those characters that
   can appear in multi-character operators. */
/* this is used by the lexical scanner (scanner.cpp) */

// bool IsOp(char c)
// {
//    return(c=='+' || c=='-' || c=='*' || c=='/' ||
//           c=='<' || c=='>' || c==':' || c=='=');
// }
/*------------------------------------------------------------*/

